% This program produces the IFE, CCEP, and CCEP-2 estimates
% for unbalanced data. It loads data that is prepared 
% in Stata.


LASTN=maxNumCompThreads(13)
parpool(10)
addpath('matlabprograms')
addpath('../mydata')
clear;
clc;

warning('off');

%LOAD DATA (exported from STATA into .xls files in the p3 program)
Y=readtable('NLSYlog_HourlyWage1_cpi.xls','ReadVariableNames',false);
Y=table2array(Y)';
X1=readtable('NLSYEducation.xls','ReadVariableNames',false);
X1=table2array(X1)';
X2=readtable('NLSYAge.xls','ReadVariableNames',false); 
X2=table2array(X2)';
X3=readtable('NLSYAge2.xls','ReadVariableNames',false); 
X3=table2array(X3)';
S=0; %Can be a Nxm matrix to assign each person into one of m groups, m<N, in order to produce group-specific, rather than person-specific, trends. Not used in this paper
[T,N]=size(Y);

%LOAD "ORIGINAL" DATA (data from Stata for regressions sometimes partials out
%covariates first, for simplicity (demo/proxy-by-time FE). But in that
%case, we still need the non-partialled data to create the correct
%cross-section averages for CCE and for calculating the marginal return. If
%no covariates are partialled out, the "original" variables won't differ
%from those above)
X1orig=readtable('NLSYEducation_orig.xls','ReadVariableNames',false);
X1orig=table2array(X1orig)';
X2orig=readtable('NLSYAge_orig.xls','ReadVariableNames',false); 
X2orig=table2array(X2orig)';
X3orig=readtable('NLSYAge2_orig.xls','ReadVariableNames',false); 
X3orig=table2array(X3orig)';
%LOAD PERSON-TIME MISSING INDICATORS (for calculating unit-specific time
%dimensions later)
Nonmissing=readtable('NonmissingObs.xls','ReadVariableNames',false); 
Nonmissing=table2array(Nonmissing)';


%SPECIFY MODEL SETTINGS
order=0; %Set the order of person/group-specific trend to be included in the model (via de-trending, before estimation) [always 0 in our paper]
k=3; %Number of regressors
kcsavg=2; %Number of cross-section averages for CCE -> number of independent variables + 1 (minus the number of age variables used as regressors if any, because age is a "d_t" variable in Pesaran paper, not an "x_it" variable)
demean=2; %Type of fixed effects to include in the model (separate from interactive fixed effects); 0-no fixed effects, 1-time period fixed effects, 2-person fixed effects, 3-two-way fixed effects
mgfe=1; %removes person FE (i.e., person intercept) in individual-level regs


%PLACE REGRESSORS TOGETHER INTO SINGLE "X" MATRIX
%NOTE: if any variables are going to be excluded from the CCE cross-section, put those in the later dimensions [e.g., age and age-squared go into dimensions 4 and 5]. 
X=zeros(T,N,k);
X(:,:,1)=X1;
X(:,:,2)=X2;
X(:,:,3)=X3;
Xorig=zeros(T,N,k);
Xorig(:,:,1)=X1orig;
Xorig(:,:,2)=X2orig;
Xorig(:,:,3)=X3orig;

%ESTIMATION
%first, demean the variables for use later
[Xdet,Ydet]=RemoveTimeTrends(X,Y,order,S); %Remove unit-specific trends time trends (no trends if order=0)
[Xdott,Ydott]=CSDemean_unbalanced(Xdet,Ydet,1); %Demean along the cross section dimension, which removes time fixed effects
[Xdoti,Ydoti]=TimeDemean_unbalanced(Xdet,Ydet,1); %Demean along the time dimension, which removes individual fixed effects
[Xdot2,Ydot2]=DemeanData_unbalanced(Xdet,Ydet); %Demean in both directions, to remove person and time fixed effects

%OLS (pooled only)
if demean==1
    Yolsp=Ydott;
    Xolsp=Xdott;
elseif demean==2
    Yolsp=Ydoti;
    Xolsp=Xdoti;
elseif demean==3
    Yolsp=Ydot2;
    Xolsp=Xdot2;
end
[betaOLSp,Uols]=Mul_panelbeta_unbalanced(Xolsp,Yolsp,eye(T));
beduc_olsp=betaOLSp(1,1)
[csdstat_ols]=CSDtest_unbalanced(Uols)


%CCE (pooled only)
[betaCCEp, se_lincom_a, Ucce, Fcce, Lcce, Fcce2, Lcce2, Fcceboot, Lcceboot] = CCEfunction_unbalanced_linear(X,Y,order,S,demean,mgfe,kcsavg,Xorig,Nonmissing); %always send in non-demeaned data; any de-meaning handled inside
beduc_ccep=betaCCEp(1,1)
%se_lincom_bs
se_lincom_a

%CCE two-step (pooled only)
[betaCCEp2step1, seCCEp2step1, Uccep21, beta_lincom_brepsCCE21, se2step_lincom1_a, nEMCCE21, se2step_lincom1_bs, Fcce2boot1, Lcce2boot1] = CCEfunction2step_unbalanced_linear(X,Y,order,S, demean, mgfe,betaCCEp,1,kcsavg,Xorig,Nonmissing); %always send in non-demeaned data
beduc_ccep2step1=betaCCEp2step1(1,1)
se2step_lincom1_bs
se2step_lincom1_a
betalincom_breps_sort=sort(beta_lincom_brepsCCE21);
CIlower_CCE21=betalincom_breps_sort(1,10)
CIupper_CCE21=betalincom_breps_sort(1,390)
[betaCCEp2step2, seCCEp2step2, Uccep22, beta_lincom_brepsCCE22, se2step_lincom2_a, nEMCCE22, se2step_lincom2_bs, Fcce2boot2, Lcce2boot2] = CCEfunction2step_unbalanced_linear(X,Y,order,S, demean, mgfe,betaCCEp,2,kcsavg,Xorig,Nonmissing); %always send in non-demeaned data
beduc_ccep2step2=betaCCEp2step2(1,1)
se2step_lincom2_bs
se2step_lincom2_a
betalincom_breps_sort=sort(beta_lincom_brepsCCE22);
CIlower_CCE22=betalincom_breps_sort(1,10)
CIupper_CCE22=betalincom_breps_sort(1,390)
[betaCCEp2step3, seCCEp2step3, Uccep23, beta_lincom_brepsCCE23, se2step_lincom3_a, nEMCCE23, se2step_lincom3_bs, Fcce2boot3, Lcce2boot3] = CCEfunction2step_unbalanced_linear(X,Y,order,S, demean, mgfe,betaCCEp,3,kcsavg,Xorig,Nonmissing); %always send in non-demeaned data
beduc_ccep2step3=betaCCEp2step3(1,1)
se2step_lincom3_bs
se2step_lincom3_a
betalincom_breps_sort=sort(beta_lincom_brepsCCE23);
CIlower_CCE23=betalincom_breps_sort(1,10)
CIupper_CCE23=betalincom_breps_sort(1,390)
[betaCCEp2step4, seCCEp2step4, Uccep24, beta_lincom_brepsCCE24, se2step_lincom4_a, nEMCCE24, se2step_lincom4_bs, Fcce2boot4, Lcce2boot4] = CCEfunction2step_unbalanced_linear(X,Y,order,S, demean, mgfe,betaCCEp,4,kcsavg,Xorig,Nonmissing); %always send in non-demeaned data
beduc_ccep2step4=betaCCEp2step4(1,1)
se2step_lincom4_bs
se2step_lincom4_a
betalincom_breps_sort=sort(beta_lincom_brepsCCE24);
CIlower_CCE24=betalincom_breps_sort(1,10)
CIupper_CCE24=betalincom_breps_sort(1,390)
save('../mydata/Results_saa2_iFE_unbal.mat')
%load('Results_saa2_iFE_all_unbal2.mat')


% %IFE (pooled only)
[betaIFE1, seIFE1, sigma21, SSR1, Uife1, nnn1, r11, betaIFE01, betaOLS1, F11, L11, VNT11, varcovIFE1, se_lincom_IFE1, beta_lincom_brepsIFE1, beta_brepsIFE1, nEMIFE1,YstarIFEbreps1,betaCCEPIFEbreps1,nnnIFEbreps1,nEMIFEbreps1,FIFEbreps1,LIFEbreps1] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,1,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg); %always send in non-demeaned data; any de-meaning handled inside
beduc_ife1=betaIFE01(1,1)
se_lincom_IFE1
betalincom_breps_sort=sort(beta_lincom_brepsIFE1);
CIlower_IFE1=betalincom_breps_sort(1,10)
CIupper_IFE1=betalincom_breps_sort(1,390)
[betaIFE2, seIFE2, sigma22, SSR2, Uife2, nnn2, r12, betaIFE02, betaOLS2, F12, L12, VNT12, varcovIFE2, se_lincom_IFE2, beta_lincom_brepsIFE2, beta_brepsIFE2, nEMIFE2,YstarIFEbreps2,betaCCEPIFEbreps2,nnnIFEbreps2,nEMIFEbreps2,FIFEbreps2,LIFEbreps2] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,2,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife2=betaIFE02(1,1)
se_lincom_IFE2
betalincom_breps_sort=sort(beta_lincom_brepsIFE2);
CIlower_IFE2=betalincom_breps_sort(1,10)
CIupper_IFE2=betalincom_breps_sort(1,390)
[betaIFE3, seIFE3, sigma23, SSR3, Uife3, nnn3, r13, betaIFE03, betaOLS3, F13, L13, VNT13, varcovIFE3, se_lincom_IFE3, beta_lincom_brepsIFE3, beta_brepsIFE3, nEMIFE3,YstarIFEbreps3,betaCCEPIFEbreps3,nnnIFEbreps3,nEMIFEbreps3,FIFEbreps3,LIFEbreps3] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,3,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife3=betaIFE03(1,1)
se_lincom_IFE3
betalincom_breps_sort=sort(beta_lincom_brepsIFE3);
CIlower_IFE3=betalincom_breps_sort(1,10)
CIupper_IFE3=betalincom_breps_sort(1,390)
[betaIFE4, seIFE4, sigma24, SSR4, Uife4, nnn4, r14, betaIFE04, betaOLS4, F14, L14, VNT14, varcovIFE4, se_lincom_IFE4, beta_lincom_brepsIFE4, beta_brepsIFE4, nEMIFE4,YstarIFEbreps4,betaCCEPIFEbreps4,nnnIFEbreps4,nEMIFEbreps4,FIFEbreps4,LIFEbreps4] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,4,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife4=betaIFE04(1,1)
se_lincom_IFE4
betalincom_breps_sort=sort(beta_lincom_brepsIFE4);
CIlower_IFE4=betalincom_breps_sort(1,10)
CIupper_IFE4=betalincom_breps_sort(1,390)
[betaIFE5, seIFE5, sigma25, SSR5, Uife5, nnn5, r15, betaIFE05, betaOLS5, F15, L15, VNT15, varcovIFE5, se_lincom_IFE5, beta_lincom_brepsIFE5, beta_brepsIFE5, nEMIFE5,YstarIFEbreps5,betaCCEPIFEbreps5,nnnIFEbreps5,nEMIFEbreps5,FIFEbreps5,LIFEbreps5] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,5,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife5=betaIFE05(1,1)
se_lincom_IFE5
betalincom_breps_sort=sort(beta_lincom_brepsIFE5);
CIlower_IFE5=betalincom_breps_sort(1,10)
CIupper_IFE5=betalincom_breps_sort(1,390)
[betaIFE6, seIFE6, sigma26, SSR6, Uife6, nnn6, r16, betaIFE06, betaOLS6, F16, L16, VTN16, varcovIFE6, se_lincom_IFE6, beta_lincom_brepsIFE6, beta_brepsIFE6, nEMIFE6,YstarIFEbreps6,betaCCEPIFEbreps6,nnnIFEbreps6,nEMIFEbreps6,FIFEbreps6,LIFEbreps6] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,6,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife6=betaIFE06(1,1)
se_lincom_IFE6
betalincom_breps_sort=sort(beta_lincom_brepsIFE6);
CIlower_IFE6=betalincom_breps_sort(1,10)
CIupper_IFE6=betalincom_breps_sort(1,390)
[betaIFE7, seIFE7, sigma27, SSR7, Uife7, nnn7, r17, betaIFE07, betaOLS7, F17, L17, VNT17, varcovIFE7, se_lincom_IFE7, beta_lincom_brepsIFE7, beta_brepsIFE7, nEMIFE7,YstarIFEbreps7,betaCCEPIFEbreps7,nnnIFEbreps7,nEMIFEbreps7,FIFEbreps7,LIFEbreps7] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,7,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife7=betaIFE07(1,1)
se_lincom_IFE7
betalincom_breps_sort=sort(beta_lincom_brepsIFE7);
CIlower_IFE7=betalincom_breps_sort(1,10)
CIupper_IFE7=betalincom_breps_sort(1,390)
[betaIFE8, seIFE8, sigma28, SSR8, Uife8, nnn8, r18, betaIFE08, betaOLS8, F18, L18, VNT18, varcovIFE8, se_lincom_IFE8, beta_lincom_brepsIFE8, beta_brepsIFE8, nEMIFE8,YstarIFEbreps8,betaCCEPIFEbreps8,nnnIFEbreps8,nEMIFEbreps8,FIFEbreps8,LIFEbreps8] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,8,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife8=betaIFE08(1,1)
se_lincom_IFE8
betalincom_breps_sort=sort(beta_lincom_brepsIFE8);
CIlower_IFE8=betalincom_breps_sort(1,10)
CIupper_IFE8=betalincom_breps_sort(1,390)
[betaIFE9, seIFE9, sigma29, SSR9, Uife9, nnn9, r19, betaIFE09, betaOLS9, F19, L19, VNT19, varcovIFE9, se_lincom_IFE9, beta_lincom_brepsIFE9, beta_brepsIFE9, nEMIFE9,YstarIFEbreps9,betaCCEPIFEbreps9,nnnIFEbreps9,nEMIFEbreps9,FIFEbreps9,LIFEbreps9] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,9,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife9=betaIFE09(1,1)
se_lincom_IFE9
betalincom_breps_sort=sort(beta_lincom_brepsIFE9);
CIlower_IFE9=betalincom_breps_sort(1,10)
CIupper_IFE9=betalincom_breps_sort(1,390)
[betaIFE10, seIFE10, sigma210, SSR10, Uife10, nnn10, r110, betaIFE010, betaOLS10, F110, L110, VNT110, varcovIFE10, se_lincom_IFE10, beta_lincom_brepsIFE10, beta_brepsIFE10, nEMIFE10,YstarIFEbreps10,betaCCEPIFEbreps10,nnnIFEbreps10,nEMIFEbreps10,FIFEbreps10,LIFEbreps10] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,10,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife10=betaIFE010(1,1)
se_lincom_IFE10
betalincom_breps_sort=sort(beta_lincom_brepsIFE10);
CIlower_IFE10=betalincom_breps_sort(1,10)
CIupper_IFE10=betalincom_breps_sort(1,390)
[betaIFE11, seIFE11, sigma211, SSR11, Uife11, nnn11, r111, betaIFE011, betaOLS11, F111, L111, VNT111, varcovIFE11, se_lincom_IFE11, beta_lincom_brepsIFE11, beta_brepsIFE11, nEMIFE11,YstarIFEbreps11,betaCCEPIFEbreps11,nnnIFEbreps11,nEMIFEbreps11,FIFEbreps11,LIFEbreps11] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,11,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife11=betaIFE011(1,1)
se_lincom_IFE11
betalincom_breps_sort=sort(beta_lincom_brepsIFE11);
CIlower_IFE11=betalincom_breps_sort(1,11)
CIupper_IFE11=betalincom_breps_sort(1,390)
[betaIFE12, seIFE12, sigma212, SSR12, Uife12, nnn12, r112, betaIFE012, betaOLS12, F112, L112, VNT112, varcovIFE12, se_lincom_IFE12, beta_lincom_brepsIFE12, beta_brepsIFE12, nEMIFE12,YstarIFEbreps12,betaCCEPIFEbreps12,nnnIFEbreps12,nEMIFEbreps12,FIFEbreps12,LIFEbreps12] = IFEfunction_unbalanced_linear(X,Y,order,S,demean,12,1,Nonmissing,Xorig,betaCCEp,mgfe,kcsavg);
beduc_ife12=betaIFE012(1,1)
se_lincom_IFE12
betalincom_breps_sort=sort(beta_lincom_brepsIFE12);
CIlower_IFE12=betalincom_breps_sort(1,12)
CIupper_IFE12=betalincom_breps_sort(1,390)
save('../mydata/Results_saa2_iFE_unbal.mat')


 
%calculate information criterion
Uife1_2=Uife1.^2;
Uife2_2=Uife2.^2;
Uife3_2=Uife3.^2;
Uife4_2=Uife4.^2;
Uife5_2=Uife5.^2;
Uife6_2=Uife6.^2;
Uife7_2=Uife7.^2;
Uife8_2=Uife8.^2;
Uife9_2=Uife9.^2;
Uife10_2=Uife10.^2;
Uife11_2=Uife11.^2;
Uife12_2=Uife12.^2;

meanUife1=nanmean(Uife1_2(:));
meanUife2=nanmean(Uife2_2(:));
meanUife3=nanmean(Uife3_2(:));
meanUife4=nanmean(Uife4_2(:));
meanUife5=nanmean(Uife5_2(:));
meanUife6=nanmean(Uife6_2(:));
meanUife7=nanmean(Uife7_2(:));
meanUife8=nanmean(Uife8_2(:));
meanUife9=nanmean(Uife9_2(:));
meanUife10=nanmean(Uife10_2(:));
meanUife11=nanmean(Uife11_2(:));
meanUife12=nanmean(Uife12_2(:));

Ucce21_2=Uccep21.^2;
Ucce22_2=Uccep22.^2;
Ucce23_2=Uccep23.^2;
Ucce24_2=Uccep24.^2;

meanUcce21=nanmean(Ucce21_2(:));
meanUcce22=nanmean(Ucce22_2(:));
meanUcce23=nanmean(Ucce23_2(:));
meanUcce24=nanmean(Ucce24_2(:));


obs=sum(Nonmissing(:));
Nbar=mean(sum(Nonmissing,2));
Tbar=mean(sum(Nonmissing,1));
gNT1=((Nbar+Tbar)/(obs))*log((obs)/(Nbar+Tbar));
NbarTbar=[Nbar Tbar];
gNT2=((Nbar+Tbar)/(obs))*log(min(NbarTbar));
gNT3=(log(min(NbarTbar))/(min(NbarTbar)));

ICp1_1_ife=log(meanUife1)+1*gNT1;
ICp1_2_ife=log(meanUife2)+2*gNT1;
ICp1_3_ife=log(meanUife3)+3*gNT1;
ICp1_4_ife=log(meanUife4)+4*gNT1;
ICp1_5_ife=log(meanUife5)+5*gNT1;
ICp1_6_ife=log(meanUife6)+6*gNT1;
ICp1_7_ife=log(meanUife7)+7*gNT1;
ICp1_8_ife=log(meanUife8)+8*gNT1;
ICp1_9_ife=log(meanUife9)+9*gNT1;
ICp1_10_ife=log(meanUife10)+10*gNT1;
ICp1_11_ife=log(meanUife11)+11*gNT1;
ICp1_12_ife=log(meanUife12)+12*gNT1;

ICp2_1_ife=log(meanUife1)+1*gNT2;
ICp2_2_ife=log(meanUife2)+2*gNT2;
ICp2_3_ife=log(meanUife3)+3*gNT2;
ICp2_4_ife=log(meanUife4)+4*gNT2;
ICp2_5_ife=log(meanUife5)+5*gNT2;
ICp2_6_ife=log(meanUife6)+6*gNT2;
ICp2_7_ife=log(meanUife7)+7*gNT2;
ICp2_8_ife=log(meanUife8)+8*gNT2;
ICp2_9_ife=log(meanUife9)+9*gNT2;
ICp2_10_ife=log(meanUife10)+10*gNT2;
ICp2_11_ife=log(meanUife11)+11*gNT2;
ICp2_12_ife=log(meanUife12)+12*gNT2;

ICp3_1_ife=log(meanUife1)+1*gNT3;
ICp3_2_ife=log(meanUife2)+2*gNT3;
ICp3_3_ife=log(meanUife3)+3*gNT3;
ICp3_4_ife=log(meanUife4)+4*gNT3;
ICp3_5_ife=log(meanUife5)+5*gNT3;
ICp3_6_ife=log(meanUife6)+6*gNT3;
ICp3_7_ife=log(meanUife7)+7*gNT3;
ICp3_8_ife=log(meanUife8)+8*gNT3;
ICp3_9_ife=log(meanUife9)+9*gNT3;
ICp3_10_ife=log(meanUife10)+10*gNT3;
ICp3_11_ife=log(meanUife11)+11*gNT3;
ICp3_12_ife=log(meanUife12)+12*gNT3;

ICp1_1_cce2=log(meanUcce21)+1*gNT1;
ICp1_2_cce2=log(meanUcce22)+2*gNT1;
ICp1_3_cce2=log(meanUcce23)+3*gNT1;
ICp1_4_cce2=log(meanUcce24)+4*gNT1;

ICp2_1_cce2=log(meanUcce21)+1*gNT2;
ICp2_2_cce2=log(meanUcce22)+2*gNT2;
ICp2_3_cce2=log(meanUcce23)+3*gNT2;
ICp2_4_cce2=log(meanUcce24)+4*gNT2;

ICp3_1_cce2=log(meanUcce21)+1*gNT3;
ICp3_2_cce2=log(meanUcce22)+2*gNT3;
ICp3_3_cce2=log(meanUcce23)+3*gNT3;
ICp3_4_cce2=log(meanUcce24)+4*gNT3;


% r selection based on minimizing change between two r values
Rselection_1=abs(beduc_ife1-beduc_ife2);
Rselection_2=abs(beduc_ife2-beduc_ife3);
Rselection_3=abs(beduc_ife3-beduc_ife4);
Rselection_4=abs(beduc_ife4-beduc_ife5);
Rselection_5=abs(beduc_ife5-beduc_ife6);
Rselection_6=abs(beduc_ife6-beduc_ife7);
Rselection_7=abs(beduc_ife7-beduc_ife8);
Rselection_8=abs(beduc_ife8-beduc_ife9);
Rselection_9=abs(beduc_ife9-beduc_ife10);
Rselection_10=abs(beduc_ife9-beduc_ife11);
Rselection_11=abs(beduc_ife9-beduc_ife12);
 
save('../mydata/Results_saa2_iFE_unbal.mat')





