/*
Performs the Gelbach decomposition using Gelbah's Stata command: b1x2

We use the Stata command for the models where the only proxies are in 
the "full model" so that we can get the standard errors -- Gelbach's 
command computes the variance-covariance matrix needed to compute the 
(linear combination) standard errors. For the models with proxies 
and loadings we bootstrap to get the standard errors.
*/


log using ./decomposition_proxiesonly_minti15.log, replace


global mydatapath "../mydata"
global outputpath "../output"

use ${mydatapath}/NLSYrep_doa.dta, clear

local sample insample_minti15
drop factor_c factor_n1 factor_n2 factor_n3
rename (factor_c_minti15 factor_n1_minti15 factor_n2_minti15 factor_n3_minti15) (factor_c factor_n1 factor_n2 factor_n3)
keep if `sample'==1

*create interactions between the proxies (and demos) and years, since b1x2 doesn't allow factor variables
qui tab year, gen(yeardum)
foreach p of varlist Sibling MotherEducation FatherEducation factor_c factor_n1 factor_n2 factor_n3 asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std {
	foreach y of varlist yeardum1-yeardum25 {
	gen `p'_x_`y'=`p'*`y'
}
}

*demean the variables; for reducing b1x2 runntime since it doesn't handle FEs well
foreach v of varlist log_HourlyWage1_cpi Education Education2 Age Age2 EducAge Sibling_x_* MotherEducation_x_* FatherEducation_x_* factor_c_x_* factor_n1_x_* factor_n2_x_* factor_n3_x_* asvab_3_1981_std_x_* asvab_4_1981_std_x_* asvab_5_1981_std_x_* asvab_6_1981_std_x_* asvab_7_1981_std_x_* asvab_8_1981_std_x_* asvab_9_1981_std_x_* asvab_10_1981_std_x_* asvab_11_1981_std_x_* asvab_12_1981_std_x_* rott_std_x_* rosen_std_x* {
	by caseid_1979: egen mi`v'=mean(`v')
	gen res`v'=`v'-mi`v'
}

*compare xtreg results from main estimation programs to results using demeaned variables; to make sure it's done correctly
sum Education
local educyears_mean=r(mean)

sum Age
local age_mean=r(mean)

xtreg log_HourlyWage1_cpi Education Education2 Age Age2 EducAge, fe cluster(caseid_1979)
lincom _b[Education]+2*_b[Education2]*`educyears_mean'+_b[EducAge]*`age_mean' 

reg reslog_HourlyWage1_cpi resEducation resEducation2 resAge resAge2 resEducAge, cluster(caseid_1979)
lincom _b[resEducation]+2*_b[resEducation2]*`educyears_mean'+_b[resEducAge]*`age_mean' 
local return=e(b)[1,1]+2*e(b)[1,2]*`educyears_mean'+e(b)[1,5]*`age_mean'

** quick test: replicate lincom's beta and se by hand; to make sure I know how to do so with b1x2 below (can't use lincom with b1x2)
display `return'
local return_var=e(V)[1,1]+e(V)[2,2]*(2^2)*(`educyears_mean'^2)+e(V)[5,5]*(`age_mean'^2)+2*2*`educyears_mean'*e(V)[2,1]+2*`age_mean'*e(V)[5,1]+2*2*`educyears_mean'*`age_mean'*e(V)[5,2]
local return_se=sqrt(abs(`return_var'))
display `return_se'
**

xtreg log_HourlyWage1_cpi Education Education2 Age Age2 EducAge c.asvab_3_1981_std#i.year c.asvab_4_1981_std#i.year c.asvab_5_1981_std#i.year c.asvab_6_1981_std#i.year c.asvab_7_1981_std#i.year c.asvab_8_1981_std#i.year c.asvab_9_1981_std#i.year c.asvab_10_1981_std#i.year c.asvab_11_1981_std#i.year c.asvab_12_1981_std#i.year c.rott_std#i.year c.rosen_std#i.year, fe cluster(caseid_1979)
lincom _b[Education]+2*_b[Education2]*`educyears_mean'+_b[EducAge]*`age_mean' 

reg reslog_HourlyWage1_cpi resEducation resEducation2 resAge resAge2 resEducAge resasvab_3_1981_std_x_* resasvab_4_1981_std_x_* resasvab_5_1981_std_x_* resasvab_6_1981_std_x_* resasvab_7_1981_std_x_* resasvab_8_1981_std_x_* resasvab_9_1981_std_x_* resasvab_10_1981_std_x_* resasvab_11_1981_std_x_* resasvab_12_1981_std_x_* resrott_std_x_* resrosen_std_x*, cluster(caseid_1979)
lincom _b[resEducation]+2*_b[resEducation2]*`educyears_mean'+_b[resEducAge]*`age_mean' 





***** woD, PCA *****
putexcel set ../output/dtable_wponly_woD_minti15.xlsx, sheet(table) replace
putexcel A1:C1, border(bottom)
putexcel B2:C2, merge hcenter
putexcel B2="OLS"
putexcel B3="PCA Proxies"
putexcel C3="Non-PCA Proxies"
putexcel A3:C3, border(bottom)
putexcel A4="Total Bias"
putexcel A5="Bias from Proxies"
putexcel A7="    Bias from Cognitive Factor"
putexcel A9="    Bias from Non-Cognitive Factor 1"
putexcel A11="    Bias from Non-Cognitive Factor 2"
putexcel A13="    Bias from Non-Cognitive Factor 3"
putexcel A15="    Bias from ASVAB Science"
putexcel A17="    Bias from ASVAB Arithmetic Reasoning"
putexcel A19="    Bias from ASVAB Mathematical Knowledge"
putexcel A21="    Bias from ASVAB Paragraph Comprehension"
putexcel A23="    Bias from ASVAB Word Knowledge"
putexcel A25="    Bias from ASVAB Coding Speed"
putexcel A27="    Bias from ASVAB Numeric Operations"
putexcel A29="    Bias from ASVAB Auto and Shop Information"
putexcel A31="    Bias from ASVAB Electronic Information"
putexcel A33="    Bias from ASVAB Mechanical Knowledge"
putexcel A35="    Bias from Rotter Scale"
putexcel A37="    Bias from Rosenberg Scale"
putexcel A38:C38, border(bottom)

*run Gelbach's b1x2
b1x2 reslog_HourlyWage1_cpi, x1all(resEducation resEducation2 resAge resAge2 resEducAge) x2all(resfactor_c_x_* resfactor_n1_x_* resfactor_n2_x_* resfactor_n3_x_*) x2delta(factor_c=resfactor_c_x_* : factor_n1=resfactor_n1_x_* : factor_n2=resfactor_n2_x_* : factor_n3=resfactor_n3_x_*)

*compute bias and standard error for the marginal return
sum Education
local educyears_mean=r(mean)

sum Age
local age_mean=r(mean)

local lincom=e(Delta)[1,5] + 2*`educyears_mean'*e(Delta)[1,10] + `age_mean'*e(Delta)[1,25]
local lincom_var=e(Covdelta)[5,5] + e(Covdelta)[10,10]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[25,25]*(`age_mean'^2) + e(Covdelta)[10,5]*2*2*`educyears_mean' + e(Covdelta)[25,5]*2*`age_mean' + e(Covdelta)[25,10]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B5=`lincom', nformat("#.####")
putexcel B6=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,1] + 2*`educyears_mean'*e(Delta)[1,6] + `age_mean'*e(Delta)[1,21]
local lincom_var=e(Covdelta)[1,1] + e(Covdelta)[6,6]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[21,21]*(`age_mean'^2) + e(Covdelta)[6,1]*2*2*`educyears_mean' + e(Covdelta)[21,1]*2*`age_mean' + e(Covdelta)[21,6]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B7=`lincom', nformat("#.####")
putexcel B8=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,2] + 2*`educyears_mean'*e(Delta)[1,7] + `age_mean'*e(Delta)[1,22]
local lincom_var=e(Covdelta)[2,2] + e(Covdelta)[7,7]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[22,22]*(`age_mean'^2) + e(Covdelta)[7,2]*2*2*`educyears_mean' + e(Covdelta)[22,2]*2*`age_mean' + e(Covdelta)[22,7]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B9=`lincom', nformat("#.####")
putexcel B10=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,3] + 2*`educyears_mean'*e(Delta)[1,8] + `age_mean'*e(Delta)[1,23]
local lincom_var=e(Covdelta)[3,3] + e(Covdelta)[8,8]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[23,23]*(`age_mean'^2) + e(Covdelta)[8,3]*2*2*`educyears_mean' + e(Covdelta)[23,3]*2*`age_mean' + e(Covdelta)[23,8]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B11=`lincom', nformat("#.####")
putexcel B12=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,4] + 2*`educyears_mean'*e(Delta)[1,9] + `age_mean'*e(Delta)[1,24]
local lincom_var=e(Covdelta)[4,4] + e(Covdelta)[9,9]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[24,24]*(`age_mean'^2) + e(Covdelta)[9,4]*2*2*`educyears_mean' + e(Covdelta)[24,4]*2*`age_mean' + e(Covdelta)[24,9]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B13=`lincom', nformat("#.####")
putexcel B14=`lincom_se', nformat("#.####")





***** woD, non-PCA *****
*run Gelbach's b1x2
b1x2 reslog_HourlyWage1_cpi, x1all(resEducation resEducation2 resAge resAge2 resEducAge) x2all(resasvab_3_1981_std_x_* resasvab_4_1981_std_x_* resasvab_5_1981_std_x_* resasvab_6_1981_std_x_* resasvab_7_1981_std_x_* resasvab_8_1981_std_x_* resasvab_9_1981_std_x_* resasvab_10_1981_std_x_* resasvab_11_1981_std_x_* resasvab_12_1981_std_x_* resrott_std_x_* resrosen_std_x*) x2delta(asvab_3=resasvab_3_1981_std_x_* : asvab_4=resasvab_4_1981_std_x_* : asvab_5=resasvab_5_1981_std_x_* : asvab_6=resasvab_6_1981_std_x_* : asvab_7=resasvab_7_1981_std_x_* : asvab_8=resasvab_8_1981_std_x_* : asvab_9=resasvab_9_1981_std_x_* : asvab_10=resasvab_10_1981_std_x_* : asvab_11=resasvab_11_1981_std_x_* : asvab_12=resasvab_12_1981_std_x_* : rott_std_x_=resrott_std_x_* : rosen_std_x=resrosen_std_x*)

*compute bias and standard error for the marginal return
sum Education
local educyears_mean=r(mean)

sum Age
local age_mean=r(mean)

local lincom=e(Delta)[1,13] + 2*`educyears_mean'*e(Delta)[1,26] + `age_mean'*e(Delta)[1,65]
local lincom_var=e(Covdelta)[13,13] + e(Covdelta)[26,26]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[65,65]*(`age_mean'^2) + e(Covdelta)[26,13]*2*2*`educyears_mean' + e(Covdelta)[65,13]*2*`age_mean' + e(Covdelta)[65,26]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C5=`lincom', nformat("#.####")
putexcel C6=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,1] + 2*`educyears_mean'*e(Delta)[1,14] + `age_mean'*e(Delta)[1,53]
local lincom_var=e(Covdelta)[1,1] + e(Covdelta)[14,14]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[53,53]*(`age_mean'^2) + e(Covdelta)[14,1]*2*2*`educyears_mean' + e(Covdelta)[53,1]*2*`age_mean' + e(Covdelta)[53,14]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C15=`lincom', nformat("#.####")
putexcel C16=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,2] + 2*`educyears_mean'*e(Delta)[1,15] + `age_mean'*e(Delta)[1,54]
local lincom_var=e(Covdelta)[2,2] + e(Covdelta)[15,15]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[54,54]*(`age_mean'^2) + e(Covdelta)[15,2]*2*2*`educyears_mean' + e(Covdelta)[54,2]*2*`age_mean' + e(Covdelta)[54,15]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C17=`lincom', nformat("#.####")
putexcel C18=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,8] + 2*`educyears_mean'*e(Delta)[1,21] + `age_mean'*e(Delta)[1,60]
local lincom_var=e(Covdelta)[8,8] + e(Covdelta)[21,21]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[60,60]*(`age_mean'^2) + e(Covdelta)[21,8]*2*2*`educyears_mean' + e(Covdelta)[60,8]*2*`age_mean' + e(Covdelta)[60,21]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C19=`lincom', nformat("#.####")
putexcel C20=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,4] + 2*`educyears_mean'*e(Delta)[1,17] + `age_mean'*e(Delta)[1,56]
local lincom_var=e(Covdelta)[4,4] + e(Covdelta)[17,17]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[56,56]*(`age_mean'^2) + e(Covdelta)[17,4]*2*2*`educyears_mean' + e(Covdelta)[56,4]*2*`age_mean' + e(Covdelta)[56,17]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C21=`lincom', nformat("#.####")
putexcel C22=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,3] + 2*`educyears_mean'*e(Delta)[1,16] + `age_mean'*e(Delta)[1,55]
local lincom_var=e(Covdelta)[3,3] + e(Covdelta)[16,16]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[55,55]*(`age_mean'^2) + e(Covdelta)[16,3]*2*2*`educyears_mean' + e(Covdelta)[55,3]*2*`age_mean' + e(Covdelta)[55,16]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C23=`lincom', nformat("#.####")
putexcel C24=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,6] + 2*`educyears_mean'*e(Delta)[1,19] + `age_mean'*e(Delta)[1,58]
local lincom_var=e(Covdelta)[6,6] + e(Covdelta)[19,19]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[58,58]*(`age_mean'^2) + e(Covdelta)[19,6]*2*2*`educyears_mean' + e(Covdelta)[58,6]*2*`age_mean' + e(Covdelta)[58,19]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C25=`lincom', nformat("#.####")
putexcel C26=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,5] + 2*`educyears_mean'*e(Delta)[1,18] + `age_mean'*e(Delta)[1,57]
local lincom_var=e(Covdelta)[5,5] + e(Covdelta)[18,18]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[57,57]*(`age_mean'^2) + e(Covdelta)[18,5]*2*2*`educyears_mean' + e(Covdelta)[57,5]*2*`age_mean' + e(Covdelta)[57,18]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C27=`lincom', nformat("#.####")
putexcel C28=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,7] + 2*`educyears_mean'*e(Delta)[1,20] + `age_mean'*e(Delta)[1,59]
local lincom_var=e(Covdelta)[7,7] + e(Covdelta)[20,20]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[59,59]*(`age_mean'^2) + e(Covdelta)[20,7]*2*2*`educyears_mean' + e(Covdelta)[59,7]*2*`age_mean' + e(Covdelta)[59,20]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C29=`lincom', nformat("#.####")
putexcel C30=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,10] + 2*`educyears_mean'*e(Delta)[1,23] + `age_mean'*e(Delta)[1,62]
local lincom_var=e(Covdelta)[10,10] + e(Covdelta)[23,23]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[62,62]*(`age_mean'^2) + e(Covdelta)[23,10]*2*2*`educyears_mean' + e(Covdelta)[62,10]*2*`age_mean' + e(Covdelta)[62,23]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C31=`lincom', nformat("#.####")
putexcel C32=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,9] + 2*`educyears_mean'*e(Delta)[1,22] + `age_mean'*e(Delta)[1,61]
local lincom_var=e(Covdelta)[9,9] + e(Covdelta)[22,22]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[61,61]*(`age_mean'^2) + e(Covdelta)[22,9]*2*2*`educyears_mean' + e(Covdelta)[61,9]*2*`age_mean' + e(Covdelta)[61,22]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C33=`lincom', nformat("#.####")
putexcel C34=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,11] + 2*`educyears_mean'*e(Delta)[1,24] + `age_mean'*e(Delta)[1,63]
local lincom_var=e(Covdelta)[11,11] + e(Covdelta)[24,24]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[63,63]*(`age_mean'^2) + e(Covdelta)[24,11]*2*2*`educyears_mean' + e(Covdelta)[63,11]*2*`age_mean' + e(Covdelta)[63,24]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C35=`lincom', nformat("#.####")
putexcel C36=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,12] + 2*`educyears_mean'*e(Delta)[1,25] + `age_mean'*e(Delta)[1,64]
local lincom_var=e(Covdelta)[12,12] + e(Covdelta)[25,25]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[64,64]*(`age_mean'^2) + e(Covdelta)[25,12]*2*2*`educyears_mean' + e(Covdelta)[64,12]*2*`age_mean' + e(Covdelta)[64,25]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C37=`lincom', nformat("#.####")
putexcel C38=`lincom_se', nformat("#.####")







***** wD, PCA *****
putexcel set ../output/dtable_wponly_wD_minti15.xlsx, sheet(table) replace
putexcel A1:C1, border(bottom)
putexcel B2:C2, merge hcenter
putexcel B2="OLS"
putexcel B3="PCA Proxies"
putexcel C3="Non-PCA Proxies"
putexcel A3:C3, border(bottom)
putexcel A4="Total Bias"
putexcel A5="Bias from Proxies"
putexcel A7="    Bias from Cognitive Factor"
putexcel A9="    Bias from Non-Cognitive Factor 1"
putexcel A11="    Bias from Non-Cognitive Factor 2"
putexcel A13="    Bias from Non-Cognitive Factor 3"
putexcel A15="    Bias from ASVAB Science"
putexcel A17="    Bias from ASVAB Arithmetic Reasoning"
putexcel A19="    Bias from ASVAB Mathematical Knowledge"
putexcel A21="    Bias from ASVAB Paragraph Comprehension"
putexcel A23="    Bias from ASVAB Word Knowledge"
putexcel A25="    Bias from ASVAB Coding Speed"
putexcel A27="    Bias from ASVAB Numeric Operations"
putexcel A29="    Bias from ASVAB Auto and Shop Information"
putexcel A31="    Bias from ASVAB Electronic Information"
putexcel A33="    Bias from ASVAB Mechanical Knowledge"
putexcel A35="    Bias from Rotter Scale"
putexcel A37="    Bias from Rosenberg Scale"
putexcel A38:C38, border(bottom)
*run Gelbach's b1x2
b1x2 reslog_HourlyWage1_cpi, x1all(resEducation resEducation2 resAge resAge2 resEducAge resSibling_x_* resMotherEducation_x_* resFatherEducation_x_*) x2all(resfactor_c_x_* resfactor_n1_x_* resfactor_n2_x_* resfactor_n3_x_*) x2delta(factor_c=resfactor_c_x_* : factor_n1=resfactor_n1_x_* : factor_n2=resfactor_n2_x_* : factor_n3=resfactor_n3_x_*)

*compute bias and standard error for the marginal return
sum Education
local educyears_mean=r(mean)

sum Age
local age_mean=r(mean)

local lincom=e(Delta)[1,5] + 2*`educyears_mean'*e(Delta)[1,10] + `age_mean'*e(Delta)[1,25]
local lincom_var=e(Covdelta)[5,5] + e(Covdelta)[10,10]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[25,25]*(`age_mean'^2) + e(Covdelta)[10,5]*2*2*`educyears_mean' + e(Covdelta)[25,5]*2*`age_mean' + e(Covdelta)[25,10]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B5=`lincom', nformat("#.####")
putexcel B6=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,1] + 2*`educyears_mean'*e(Delta)[1,6] + `age_mean'*e(Delta)[1,21]
local lincom_var=e(Covdelta)[1,1] + e(Covdelta)[6,6]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[21,21]*(`age_mean'^2) + e(Covdelta)[6,1]*2*2*`educyears_mean' + e(Covdelta)[21,1]*2*`age_mean' + e(Covdelta)[21,6]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B7=`lincom', nformat("#.####")
putexcel B8=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,2] + 2*`educyears_mean'*e(Delta)[1,7] + `age_mean'*e(Delta)[1,22]
local lincom_var=e(Covdelta)[2,2] + e(Covdelta)[7,7]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[22,22]*(`age_mean'^2) + e(Covdelta)[7,2]*2*2*`educyears_mean' + e(Covdelta)[22,2]*2*`age_mean' + e(Covdelta)[22,7]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B9=`lincom', nformat("#.####")
putexcel B10=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,3] + 2*`educyears_mean'*e(Delta)[1,8] + `age_mean'*e(Delta)[1,23]
local lincom_var=e(Covdelta)[3,3] + e(Covdelta)[8,8]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[23,23]*(`age_mean'^2) + e(Covdelta)[8,3]*2*2*`educyears_mean' + e(Covdelta)[23,3]*2*`age_mean' + e(Covdelta)[23,8]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B11=`lincom', nformat("#.####")
putexcel B12=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,4] + 2*`educyears_mean'*e(Delta)[1,9] + `age_mean'*e(Delta)[1,24]
local lincom_var=e(Covdelta)[4,4] + e(Covdelta)[9,9]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[24,24]*(`age_mean'^2) + e(Covdelta)[9,4]*2*2*`educyears_mean' + e(Covdelta)[24,4]*2*`age_mean' + e(Covdelta)[24,9]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel B13=`lincom', nformat("#.####")
putexcel B14=`lincom_se', nformat("#.####")





***** woD, non-PCA *****
*run Gelbach's b1x2
b1x2 reslog_HourlyWage1_cpi, x1all(resEducation resEducation2 resAge resAge2 resEducAge resSibling_x_* resMotherEducation_x_* resFatherEducation_x_*) x2all(resasvab_3_1981_std_x_* resasvab_4_1981_std_x_* resasvab_5_1981_std_x_* resasvab_6_1981_std_x_* resasvab_7_1981_std_x_* resasvab_8_1981_std_x_* resasvab_9_1981_std_x_* resasvab_10_1981_std_x_* resasvab_11_1981_std_x_* resasvab_12_1981_std_x_* resrott_std_x_* resrosen_std_x*) x2delta(asvab_3=resasvab_3_1981_std_x_* : asvab_4=resasvab_4_1981_std_x_* : asvab_5=resasvab_5_1981_std_x_* : asvab_6=resasvab_6_1981_std_x_* : asvab_7=resasvab_7_1981_std_x_* : asvab_8=resasvab_8_1981_std_x_* : asvab_9=resasvab_9_1981_std_x_* : asvab_10=resasvab_10_1981_std_x_* : asvab_11=resasvab_11_1981_std_x_* : asvab_12=resasvab_12_1981_std_x_* : rott_std_x_=resrott_std_x_* : rosen_std_x=resrosen_std_x*)

*compute bias and standard error for the marginal return
sum Education
local educyears_mean=r(mean)

sum Age
local age_mean=r(mean)

local lincom=e(Delta)[1,13] + 2*`educyears_mean'*e(Delta)[1,26] + `age_mean'*e(Delta)[1,65]
local lincom_var=e(Covdelta)[13,13] + e(Covdelta)[26,26]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[65,65]*(`age_mean'^2) + e(Covdelta)[26,13]*2*2*`educyears_mean' + e(Covdelta)[65,13]*2*`age_mean' + e(Covdelta)[65,26]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C5=`lincom', nformat("#.####")
putexcel C6=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,1] + 2*`educyears_mean'*e(Delta)[1,14] + `age_mean'*e(Delta)[1,53]
local lincom_var=e(Covdelta)[1,1] + e(Covdelta)[14,14]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[53,53]*(`age_mean'^2) + e(Covdelta)[14,1]*2*2*`educyears_mean' + e(Covdelta)[53,1]*2*`age_mean' + e(Covdelta)[53,14]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C15=`lincom', nformat("#.####")
putexcel C16=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,2] + 2*`educyears_mean'*e(Delta)[1,15] + `age_mean'*e(Delta)[1,54]
local lincom_var=e(Covdelta)[2,2] + e(Covdelta)[15,15]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[54,54]*(`age_mean'^2) + e(Covdelta)[15,2]*2*2*`educyears_mean' + e(Covdelta)[54,2]*2*`age_mean' + e(Covdelta)[54,15]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C17=`lincom', nformat("#.####")
putexcel C18=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,8] + 2*`educyears_mean'*e(Delta)[1,21] + `age_mean'*e(Delta)[1,60]
local lincom_var=e(Covdelta)[8,8] + e(Covdelta)[21,21]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[60,60]*(`age_mean'^2) + e(Covdelta)[21,8]*2*2*`educyears_mean' + e(Covdelta)[60,8]*2*`age_mean' + e(Covdelta)[60,21]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C19=`lincom', nformat("#.####")
putexcel C20=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,4] + 2*`educyears_mean'*e(Delta)[1,17] + `age_mean'*e(Delta)[1,56]
local lincom_var=e(Covdelta)[4,4] + e(Covdelta)[17,17]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[56,56]*(`age_mean'^2) + e(Covdelta)[17,4]*2*2*`educyears_mean' + e(Covdelta)[56,4]*2*`age_mean' + e(Covdelta)[56,17]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C21=`lincom', nformat("#.####")
putexcel C22=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,3] + 2*`educyears_mean'*e(Delta)[1,16] + `age_mean'*e(Delta)[1,55]
local lincom_var=e(Covdelta)[3,3] + e(Covdelta)[16,16]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[55,55]*(`age_mean'^2) + e(Covdelta)[16,3]*2*2*`educyears_mean' + e(Covdelta)[55,3]*2*`age_mean' + e(Covdelta)[55,16]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C23=`lincom', nformat("#.####")
putexcel C24=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,6] + 2*`educyears_mean'*e(Delta)[1,19] + `age_mean'*e(Delta)[1,58]
local lincom_var=e(Covdelta)[6,6] + e(Covdelta)[19,19]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[58,58]*(`age_mean'^2) + e(Covdelta)[19,6]*2*2*`educyears_mean' + e(Covdelta)[58,6]*2*`age_mean' + e(Covdelta)[58,19]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C25=`lincom', nformat("#.####")
putexcel C26=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,5] + 2*`educyears_mean'*e(Delta)[1,18] + `age_mean'*e(Delta)[1,57]
local lincom_var=e(Covdelta)[5,5] + e(Covdelta)[18,18]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[57,57]*(`age_mean'^2) + e(Covdelta)[18,5]*2*2*`educyears_mean' + e(Covdelta)[57,5]*2*`age_mean' + e(Covdelta)[57,18]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C27=`lincom', nformat("#.####")
putexcel C28=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,7] + 2*`educyears_mean'*e(Delta)[1,20] + `age_mean'*e(Delta)[1,59]
local lincom_var=e(Covdelta)[7,7] + e(Covdelta)[20,20]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[59,59]*(`age_mean'^2) + e(Covdelta)[20,7]*2*2*`educyears_mean' + e(Covdelta)[59,7]*2*`age_mean' + e(Covdelta)[59,20]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C29=`lincom', nformat("#.####")
putexcel C30=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,10] + 2*`educyears_mean'*e(Delta)[1,23] + `age_mean'*e(Delta)[1,62]
local lincom_var=e(Covdelta)[10,10] + e(Covdelta)[23,23]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[62,62]*(`age_mean'^2) + e(Covdelta)[23,10]*2*2*`educyears_mean' + e(Covdelta)[62,10]*2*`age_mean' + e(Covdelta)[62,23]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C31=`lincom', nformat("#.####")
putexcel C32=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,9] + 2*`educyears_mean'*e(Delta)[1,22] + `age_mean'*e(Delta)[1,61]
local lincom_var=e(Covdelta)[9,9] + e(Covdelta)[22,22]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[61,61]*(`age_mean'^2) + e(Covdelta)[22,9]*2*2*`educyears_mean' + e(Covdelta)[61,9]*2*`age_mean' + e(Covdelta)[61,22]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C33=`lincom', nformat("#.####")
putexcel C34=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,11] + 2*`educyears_mean'*e(Delta)[1,24] + `age_mean'*e(Delta)[1,63]
local lincom_var=e(Covdelta)[11,11] + e(Covdelta)[24,24]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[63,63]*(`age_mean'^2) + e(Covdelta)[24,11]*2*2*`educyears_mean' + e(Covdelta)[63,11]*2*`age_mean' + e(Covdelta)[63,24]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C35=`lincom', nformat("#.####")
putexcel C36=`lincom_se', nformat("#.####")

local lincom=e(Delta)[1,12] + 2*`educyears_mean'*e(Delta)[1,25] + `age_mean'*e(Delta)[1,64]
local lincom_var=e(Covdelta)[12,12] + e(Covdelta)[25,25]*(2^2)*(`educyears_mean'^2) + e(Covdelta)[64,64]*(`age_mean'^2) + e(Covdelta)[25,12]*2*2*`educyears_mean' + e(Covdelta)[64,12]*2*`age_mean' + e(Covdelta)[64,25]*2*2*`educyears_mean'*`age_mean'
local lincom_se=sqrt(abs(`lincom_var'))
putexcel C37=`lincom', nformat("#.####")
putexcel C38=`lincom_se', nformat("#.####")


log close

