
% Produces a 2-step verison of Pesaran's (2006) estimator with bootstrapped
% standard errors for unbalanced data.
% Rather than using cross-section averages as the proxies, it uses 
% principal component factors applied to CCE y-b*x residuals. 

% Note, this program computes both an analytical version of standard errors 
% and a bootstrapped version. The bootstrap is written within this program.

% X1, Y1 = data to use for regression
%	note, X1 is TxNxp. If any of the p variables in X1 are to be excluded from use as cross-section averages in the CCE estimator, 
%	then these variables should be included last in the order the 1,2,...p variables loaded into X1.
% order = order of group-specific time trends (NOTE-always 0, i.e. no trends, for this returns to schooling project)
% S = places units into groups for time trends, if trends aren't at unit level (NOTE-also not relevant for returns to schooling project)
% demean - 0=no demeaning for fixed effects; 1=cross-sectionally demeans to remove time FE, 2=demeans across time to remove unit FE; 3=both
% mgfe - model includes unit FE? (0/1)
% betaCCEp - CCE estimate obtained from the CCEfunction program
% r2step - number of factors assumed in the 2step CCE estimator
% knum - number of variables whose cross-section averages will be included in the CCE proxies (knum<=p)
% Xorig - original X data. Needed if X1 above has already partialled out any covariates. If X1 above is the original data, then just re-use X1 here.
% nonmiss - TxN 0/1 matrix; 1 if unit-time observation is non-missing; 0 if missing


function [betaCCEp2step, seCCEp2step, Uccep2defac, beta2step_lincom_breps, se_cce2lincom_a, nEMCCE, se2step_lincom, Fcceboot, Lcceboot] = CCEfunction2step_unbalanced_linear(X1,Y1,order,S, demean, mgfe,betaCCEp,r2step,knum,Xorig,nonmiss); 
[T,N,p]=size(X1);
Ti=sum(nonmiss,1);

% remove state-specific trends and demean for X,Y in estimation
[Xdet,Ydet] = RemoveTimeTrends(X1,Y1,order,S);
if demean==0
    Xdot=Xdet;
    Ydot=Ydet;
elseif demean==1
    [Xdot,Ydot]=CSDemean_unbalanced(Xdet,Ydet,1);
elseif demean==2
    [Xdot,Ydot]=TimeDemean_unbalanced(Xdet,Ydet,1);
elseif demean==3
    [Xdot,Ydot]=DemeanData_unbalanced(Xdet,Ydet);
end
X=Xdot;
Y=Ydot;
% Prepare data used for cross-section averages; not necessarily the
% same as above because above may have partialled out time-related
% effects (e.g., demo-by-year) or have regular time fixed effects, in
% which case the cross-section averages are ~0. Person fixed effects
% are handled here, but could alternatively be handled via a Tx1 vector
% of ones in the Zw matrix below.
if mgfe==1
    [Xcsavgs,Ycsavgs]=TimeDemean_unbalanced(Xorig,Y1,1);
elseif mgfe==0
    Xcsavgs=Xorig;
    Ycsavgs=Y1;
end


Zw=zeros(T,knum);
Zw(:,1)=nanmean(Ycsavgs,2);
for j=1:knum-1;
    Zw(:,1+j)=nanmean(Xcsavgs(:,:,j),2);
end;



% estimate factors from CCE residuals
Uccep=Y;
for j=1:p
    Uccep=Uccep-X(:,:,j)* betaCCEp(j,1);
end
li=zeros(knum,N);
for i=1:N;
    Ui=Uccep(:,i);

    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(Ui));
    for j=1:knum-1;
        zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(Ui));
    end;
    Hwi=Zwi;

    Ui=Ui(~isnan(Ui));

    Hwi_mean=mean(Hwi,1);
    Hwi_d=Hwi-ones(Ti(1,i),1)*Hwi_mean;
    li(:,i)=(Hwi'*Hwi)\(Hwi'*Ui);
end;
F1=Zw;
L1=li';

[Fcce,Lcce,VNTcce,nEMCCE]=panelFactorNew_EM(Uccep,r2step,0.005,F1,L1);




% Estimation
% Pooled Estimator
[XXinv]= Mul_XXinv_unbalanced(X);
betaCCEp2step=Mul_panelbetaNew_unbalanced(X,XXinv,Y,Fcce,Lcce);




% standard error - CCEp
%Asymptotic Variance estimate from ZZ%
Uccep2=Y;
for j=1:p
    Uccep2=Uccep2-X(:,:,j)* betaCCEp2step(j,1);
end

Lccenew=zeros(r2step,N);
for i=1:N
    yi=Y(:,i);

    Fi=zeros(Ti(1,i),r2step);
    fi1=Fcce(:,1);
    Fi(:,1)=fi1(~isnan(yi));
    for j=1:r2step-1
	fij=Fcce(:,1+j);
        Fi(:,1+j)=fij(~isnan(yi));
    end;
    Uccepi=Uccep2(:,i);
    Uccepi=Uccepi(~isnan(Uccepi));
    Lccenew(:,i)=(Fi'*Fi)\(Fi'*Uccepi);
end

Uccep2defac=Uccep2 - Fcce*Lccenew;

XX=zeros(p,p,N);
sighati=zeros(p,p,N);
for i=1:N
     Xi=zeros(Ti(1,i),p);
     for j=1:p
          xij=X(:,i,j);
	  Xi(:,j)=xij(~isnan(xij));
     end;

     Uccep2defaci=Uccep2defac(:,i);
     Uccep2defaci=Uccep2defaci(~isnan(Uccep2defaci));
     sighati(:,:,i)=Xi'*Uccep2defaci*Uccep2defaci'*Xi;

     yi=Y(:,i);
     Fi=zeros(Ti(1,i),r2step);
     fi1=Fcce(:,1);
     Fi(:,1,1)=fi1(~isnan(yi));
     for j=1:r2step-1
         fij=Fcce(:,1+j);
         Fi(:,1+j)=fij(~isnan(yi));
     end;
     Hwi=Fi;
     Mi=eye(Ti(1,i)) - Hwi*((Hwi'*Hwi)\Hwi');
     XX(:,:,i)=(1/N)*(Xi'*Mi*Xi);
end;
XXsum=sum(XX,3);

sighat=(1/N)*sum(sighati,3);
vhat=(XXsum\sighat)*(XXsum\eye(p));

x1=Xorig(:,:,1);
%x4=Xorig(:,:,4);
%A=[1];
%var_cce2lincom_a=(1/N)*A*vhat*A';
%se_cce2lincom_a=sqrt(var_cce2lincom_a);
se_cce2lincom_a=sqrt((1/N)*vhat(1,1));


[betaCCEp2step_breps,YstarCCE2stepbreps,Fcceboot,Lcceboot]=CCE2stepbc_se_bootstrap(betaCCEp,X,Y,X1,Y1,Uccep,r2step,order,S, demean, Xorig,nonmiss,mgfe,knum,Zw);
[T,N,p]=size(X1);

seCCEp2step=std(betaCCEp2step_breps,0,2);
beta2step_lincom_breps=zeros(1,400);
for b=1:400
    beta2step_lincom_breps(1,b)=betaCCEp2step_breps(1,b);
end
se2step_lincom=std(beta2step_lincom_breps);

beta2step_lincom=betaCCEp2step(1,1);



