
% Produces Pesaran's (2006) pooled estimator extended to unbalanced data as in Zhou and Zhang (2016).

% X1, Y1 = data to use for regression
%	note, X1 is TxNxp. If any of the p variables in X1 are to be excluded from use as cross-section averages in the CCE estimator, 
%	then these variables should be included last in the order the 1,2,...p variables loaded into X1.
% order = order of group-specific time trends (NOTE-always 0, i.e. no trends, for this returns to schooling project)
% S = places units into groups for time trends, if trends aren't at unit level (NOTE-also not relevant for returns to schooling project)
% demean - 0=no demeaning for fixed effects; 1=cross-sectionally demeans to remove time FE, 2=demeans across time to remove unit FE; 3=both
% mgfe - model includes unit FE? (0/1)
% knum - number of variables whose cross-section averages will be included in the CCE proxies (knum<=p)
% Xorig - original X data. Needed if X1 above has already partialled out any covariates. If X1 above is the original data, then just re-use X1 here.
% nonmiss - TxN 0/1 matrix; 1 if unit-time observation is non-missing; 0 if missing


function [betaCCEp,se_ccelincom_a,Uccep_defac,Fcce,Lcce,Fcce2,Lcce2,Fcceboot,Lcceboot] = CCEfunction_unbalanced_linear(X1,Y1,order,S, demean, mgfe, knum, Xorig, nonmiss); 
[T,N,p]=size(X1);
Ti=sum(nonmiss,1);

% remove state-specific trends and demean for X,Y in estimation
[Xdet,Ydet] = RemoveTimeTrends(X1,Y1,order,S);
if demean==0
    Xdot=Xdet;
    Ydot=Ydet;
elseif demean==1
    [Xdot,Ydot]=CSDemean_unbalanced(Xdet,Ydet,1);
elseif demean==2
    [Xdot,Ydot]=TimeDemean_unbalanced(Xdet,Ydet,1);
elseif demean==3
    [Xdot,Ydot]=DemeanData_unbalanced(Xdet,Ydet);
end
X=Xdot;
Y=Ydot;
% Prepare data used for cross-section averages; not necessarily the
% same as above because above may have partialled out time-related
% effects (e.g., demo-by-year) or have regular time fixed effects, in
% which case the cross-section averages are ~0. Person fixed effects
% are handled here, but could alternatively be handled via a Tx1 vector
% of ones in the Zw matrix below.
if mgfe==1
    [Xcsavgs,Ycsavgs]=TimeDemean_unbalanced(Xorig,Y1,1);
elseif mgfe==0
    Xcsavgs=Xorig;
    Ycsavgs=Y1;
end


% Estimation
% Pooled Estimator
%Create the H-bar matrix -a proxy for the factors- and the M-bar matrix from Pesaran (2006)
% beta - CCEp
Zw=zeros(T,knum);
Zw(:,1)=nanmean(Ycsavgs,2);
for j=1:knum-1;
    Zw(:,1+j)=nanmean(Xcsavgs(:,:,j),2);
end;


XX=zeros(p,p,N);
XY=zeros(p,1,N);
for i=1:N;
    Xi=zeros(Ti(1,i),p);
    for j=1:p;
        xij=X(:,i,j);
        Xi(:,j)=xij(~isnan(xij));
    end;
    yi=Y(:,i);
    Yi=yi(~isnan(yi)); 
    
    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(yi));
    for j=1:knum-1;
	zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(yi));
    end;
    Hwi=Zwi;
    Mi=eye(Ti(1,i)) - Hwi*((Hwi'*Hwi)\Hwi');

    XX(:,:,i)=(1/N)*(Xi'*Mi*Xi);
    XY(:,:,i)=(1/N)*(Xi'*Mi*Yi);
end;
XXsum=sum(XX,3);
XYsum=sum(XY,3);
betaCCEp=XXsum\XYsum;




% standard error - CCEp
Uccep=Y;
for j=1:p;
    Uccep=Uccep-X(:,:,j)* betaCCEp(j,1);
end


Lcce=zeros(knum,N);

sighati=zeros(p,p,N);

for i=1:N;
    Xi=zeros(Ti(1,i),p);
    for j=1:p
        xij=X(:,i,j);
        Xi(:,j)=xij(~isnan(xij));
    end;
    
    yi=Y(:,i);
    Yi=yi(~isnan(yi)); 
    Uccepi=Yi-Xi*betaCCEp;
    
    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(yi));
    for j=1:knum-1;
	zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(yi));
    end;
    Hwi=Zwi;
    Mi=eye(Ti(1,i)) - Hwi*((Hwi'*Hwi)\Hwi');
    
    Uccep_defaci=Mi*Uccepi;
    sighati(:,:,i)=Xi'*Uccep_defaci*Uccep_defaci'*Xi;

    Hwi_mean=mean(Hwi,1);
    Hwi_d=Hwi-ones(Ti(1,i),1)*Hwi_mean;
    li(:,i)=(Hwi'*Hwi)\(Hwi'*Uccepi); 
end
F1=Zw;
L1=li';
Uccep_defac=Uccep-F1*L1';


% estimate factors from CCE residuals
if mgfe==1
    [Xcsavgs,Ycsavgs]=TimeDemean_unbalanced(Xorig,Y1,1);
elseif mgfe==0
    Xcsavgs=Xorig;
    Ycsavgs=Y1;
end
Zw=zeros(T,knum);
Zw(:,1)=nanmean(Ycsavgs,2);
for j=1:knum-1;
    Zw(:,1+j)=nanmean(Xcsavgs(:,:,j),2);
end;
Uccep=Y;
for j=1:p
    Uccep=Uccep-X(:,:,j)* betaCCEp(j,1);
end
Ti=sum(nonmiss,1);
li=zeros(knum,N);
for i=1:N;
    Ui=Uccep(:,i);

    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(Ui));
    for j=1:knum-1;
        zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(Ui));
    end;
    Hwi=Zwi;

    Ui=Ui(~isnan(Ui));

    Hwi_mean=mean(Hwi,1);
    Hwi_d=Hwi-ones(Ti(1,i),1)*Hwi_mean;
    li(:,i)=(Hwi'*Hwi)\(Hwi'*Ui);
end;
Fcce=Zw;
Lcce=li';

[Fcce2,Lcce2,VNTcce,nEMCCE]=panelFactorNew_EM(Uccep,4,0.005,Fcce,Lcce);




%Asymptotic Variance estimate from ZZ%
sighat=(1/N)*sum(sighati,3);
vhat=(XXsum\sighat)*(XXsum\eye(p));

%x1=Xorig(:,:,1);
%x4=Xorig(:,:,4);
%A=[1];
%var_ccelincom_a=(1/N)*A*vhat*A';
%se_ccelincom_a=sqrt(var_ccelincom_a);
se_ccelincom_a=sqrt((1/N)*vhat(1,1));



% NOW, PERFORM BOOTSTRAP (will be used later for decomposition CIs)
% GENERATE CONSTANT PARTS OF BOOSTRAP REPS
Ybar_i=nanmean(Y1,1);
Xbar_i=zeros(p,N);
for j=1:p;
   Xbar_i(j,:)=nanmean(X1(:,:,j),1);
end
%alpha_hat=Ybar_i'-Xbar_i'*betaCCEp-mew_hat*ones(N,1);
alpha_hat=Ybar_i'-Xbar_i'*betaCCEp;

y_star=ones(T,1)*alpha_hat';
for j=1:p;
   y_star=y_star+X1(:,:,j)*betaCCEp(j,1);
end
Hw=Zw;
y_star=y_star+Hw*Lcce';


% LOOP THROUGH BOOTSTRAP REPS, GENERATE DATA, AND ESTIMATE COEFS
Fcceboot=zeros(T,knum,400);
Lcceboot=zeros(N,knum,400);

rng('default');
breps=400;
betaCCEp_breps=zeros(p,breps);
beta_lincom_breps=zeros(1,breps);
for b=1:breps
   w=normrnd(0,1,1,N);
   W=repmat(w,T,1);
   U_star=Uccep_defac.*W;
   Y_star=y_star+U_star;


% remove state-specific trends and demean for X,Y in estimation
[Xdet,Ydet] = RemoveTimeTrends(X1,Y_star,order,S);
if demean==0
    Xdot=Xdet;
    Ydot=Ydet;
elseif demean==1
    [Xdot,Ydot]=CSDemean_unbalanced(Xdet,Ydet,1);
elseif demean==2
    [Xdot,Ydot]=TimeDemean_unbalanced(Xdet,Ydet,1);
elseif demean==3
    [Xdot,Ydot]=DemeanData_unbalanced(Xdet,Ydet);
end
X=Xdot;
Y=Ydot;
% % Prepare data used for cross-section averages; not necessarily the
% % same as above because above may have partialled out time-related
% % effects (e.g., demo-by-year) or have regular time fixed effects, in
% % which case the cross-section averages are ~0. Person fixed effects
% % are handled here, but could alternatively be handled via a Tx1 vector
% % of ones in the Zw matrix below.
if mgfe==1
     [Xcsavgs,Ycsavgs]=TimeDemean_unbalanced(Xorig,Y_star,1);
 elseif mgfe==0
     Xcsavgs=Xorig;
     Ycsavgs=Y_star;
 end

 % Estimation
 % Pooled Estimator
 %Create the H-bar matrix -a proxy for the factors- and the M-bar matrix from Pesaran (2006)
 Zw=zeros(T,knum);
 Zw(:,1)=nanmean(Ycsavgs,2);
 for j=1:knum-1
     Zw(:,1+j)=nanmean(Xcsavgs(:,:,j),2);
 end;

XX=zeros(p,p,N);
XY=zeros(p,1,N);
for i=1:N;
    Xi=zeros(Ti(1,i),p);
    for j=1:p;
        xij=X(:,i,j);
        Xi(:,j)=xij(~isnan(xij));
    end;
    yi=Y(:,i);
    Yi=yi(~isnan(yi));

    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(yi));
    for j=1:knum-1;
        zwij=Zw(:,j+1);
        Zwi(:,1+j)=zwij(~isnan(yi));
    end;
    Hwi=Zwi;
    Mi=eye(Ti(1,i)) - Hwi*((Hwi'*Hwi)\Hwi');

    XX(:,:,i)=(1/N)*(Xi'*Mi*Xi);
    XY(:,:,i)=(1/N)*(Xi'*Mi*Yi);
end;
XXsum=sum(XX,3);
XYsum=sum(XY,3);
betaCCEp_breps(:,b)=XXsum\XYsum;


Uccepb=Y;
for j=1:p
    Uccepb=Uccepb-X(:,:,j)* betaCCEp_breps(j,1);
end
li=zeros(knum,N);
for i=1:N;
    Ui=Uccepb(:,i);

    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(Ui));
    for j=1:knum-1;
        zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(Ui));
    end;
    Hwi=Zwi;

    Ui=Ui(~isnan(Ui));

    Hwi_mean=mean(Hwi,1);
    Hwi_d=Hwi-ones(Ti(1,i),1)*Hwi_mean;
    li(:,i)=(Hwi'*Hwi)\(Hwi'*Ui);
end;
Fcceb=Zw;
Lcceb=li';
Fcceboot(:,:,b)=Fcceb;
Lcceboot(:,:,b)=Lcceb;

%[Fcceb,Lcceb,VNTcce,nEMCCE]=panelFactorNew_EMalt(Uccepb,4,0.005,Fcceb,Lcceb);


beta_breps(1,b)=betaCCEp_breps(1,b);

end

seCCEp=std(betaCCEp_breps,0,2);
se=std(beta_breps);



