
% This program performs Interactive Fixed Effects (IFE) estimation 
% as developed in Bai (2009) for unbalanced data.
% Bias-correction and standard errors are computed via bootstrap. The 
% bootstrap is performed in a separate program called within this program.

% X1, Y1 = data to use for regression
%	note, X1 is TxNxp. If any of the p variables in X1 are to be excluded from use as cross-section averages in the CCE estimator, 
%	then these variables should be included last in the order the 1,2,...p variables loaded into X1.
% order = order of group-specific time trends (NOTE-always 0, i.e. no trends, for this returns to schooling project)
% S = places units into groups for time trends, if trends aren't at unit level (NOTE-also not relevant for returns to schooling project)
% demean - 0=no demeaning for fixed effects; 1=cross-sectionally demeans to remove time FE, 2=demeans across time to remove unit FE; 3=both
% rmax - maximum number of possible factors if using Bai and Ng (2002) criterion (rfix=0), or the exact number of factors if not (rfix=1)
% rfix - fix the number of factors at rmax? (0/1)
% nonmiss - TxN 0/1 matrix; 1 if unit-time observation is non-missing; 0 if missing
% Xorig - original X data. Needed if X1 above has already partialled out any covariates. If X1 above is the original data, then just re-use X1 here.
% mgfe - model includes unit FE? (0/1)
% knum - number of variables whose cross-section averages will be included in the CCE proxies (knum<=p)


function [betaIFE, seIFE, sigma2, SSR, e1nan, nnn, r1, betaIFE0, betaOLS, F1, L1, VNT1, varcovIFE,se_lincom,beta1_lincom_breps,betaIFE1breps,nEMIFE,YstarIFEbreps,betaCCEPIFEbreps,nnnIFEbreps,nEMIFEbreps,FIFEbreps,LIFEbreps] = IFEfunction_unbalanced(X1,Y1,order,S,demean,rmax,rfix,nonmiss,Xorig,betaCCEp,mgfe,knum); 
[T,N,p]=size(X1);
Ti=sum(nonmiss,1);
obs=sum(nonmiss(:));
Nbar=mean(sum(nonmiss,2));
Tbar=mean(sum(nonmiss,1));

% remove state-specific trends
[Xdet,Ydet] = RemoveTimeTrends(X1,Y1,order,S);
% remove fixed effects
if demean==0
    Xdot=Xdet;
    Ydot=Ydet;
elseif demean==1
    [Xdot,Ydot]=CSDemean_unbalanced(Xdet,Ydet,1);
elseif demean==2
    [Xdot,Ydot]=TimeDemean_unbalanced(Xdet,Ydet,1);
elseif demean==3
    [Xdot,Ydot]=DemeanData_unbalanced(Xdet,Ydet);
end
Y=Ydot;
X=Xdot;

% calculate XXinv outside of loop to save time
[XXinv]= Mul_XXinv_unbalanced(X); 


% Estimation
betaIFE0=zeros(p,2);
betaIFE=zeros(p,2);    % to contain the interative effect estimators, with different staring methods (2 methods)
nnn=zeros(1,2);   % contain the number of iterations for the 2 methods to achieve convergence
sigma2=zeros(1,2); % the estimated residual variance for each starting method, also the optimal value of the objective function
varcovIFE=zeros(p,p,2);  % contains the IFE variance-covariance matrix for the two starting methods
seIFE=zeros(p,2);     % contains the IFE standard errors for the 2 different starting methods
SSR=zeros(1,2);
betaOLS=zeros(p,1);

% Starting method 1:
% Initiate IFE by using the one-step CCE results (obtain first F,L estimates from CCE residuals)
U=Y;
for k=1:p;
    U=U-X(:,:,k)* betaCCEp(k,1);
end

% Get the first estimate of r and F,lambda (because iteration below starts by estimating beta)
if rfix==0
    [r1,~,~,~]=nbplog(U,rmax,1,0);
elseif rfix==1
    r1=rmax;
end
li=zeros(knum,N);
if mgfe==1
    [Xcsavgs,Ycsavgs]=TimeDemean_unbalanced(Xorig,Y1,1);
elseif mgfe==0
    Xcsavgs=Xorig;
    Ycsavgs=Y1;
end
Zw=zeros(T,knum);
Zw(:,1)=nanmean(Ycsavgs,2);
for j=1:knum-1;
    Zw(:,1+j)=nanmean(Xcsavgs(:,:,j),2);
end;
for i=1:N;
    Ui=U(:,i);

    Zwi=zeros(Ti(1,i),knum);
    zwi1=Zw(:,1);
    Zwi(:,1)=zwi1(~isnan(Ui));
    for j=1:knum-1;
        zwij=Zw(:,1+j);
        Zwi(:,1+j)=zwij(~isnan(Ui));
    end;
    Hwi=[Zwi];

    Ui=Ui(~isnan(Ui));

    Hwi_mean=mean(Hwi,1);
    Hwi_d=Hwi-ones(Ti(1,i),1)*Hwi_mean;
    
    li(:,i)=(Hwi'*Hwi)\(Hwi'*Ui);
end;
F1=[Zw];
L1=li';


% Perform IFE iteration
[betaIFE0(:,1), F1,L1, VNT1, e1, nnn(1,1),r1,nEMIFE]=Mul_betaIterNew_unbalanced(X, Y, XXinv, U,F1,L1, r1,rmax,rfix, 0.005,nonmiss);
e1nan=e1;
e1(isnan(e1))=0;
sigma2(1,1)=trace(e1*e1')/(obs-r1*(Nbar+Tbar)+r1^2-2);
SSR(1,1)=trace(e1*e1');



[betaIFE1breps,YstarIFEbreps,betaCCEPIFEbreps,nnnIFEbreps,nEMIFEbreps,FIFEbreps,LIFEbreps]=IFEbc_se_bootstrap(betaIFE0,X1,Y1,XXinv,e1nan,F1,L1,rmax,rfix,order,S, demean, Xorig, nonmiss,mgfe,knum,Zw);

meanbetabcIFE1breps=mean(betaIFE1breps,2);
betaIFE(:,1)=betaIFE0(:,1) - (meanbetabcIFE1breps - betaIFE0(:,1));

seIFE(:,1)=std(betaIFE1breps,0,2);
Xj1=Xorig(:,:,1);
Xj4=Xorig(:,:,4);
 se_lincom=zeros(1,2);
beta1_lincom_breps=zeros(1,400);
for b=1:400
    beta1_lincom_breps(1,b)=betaIFE1breps(1,b)+2*nanmean(Xj1(:))*betaIFE1breps(2,b)+betaIFE1breps(3,b)*nanmean(Xj4(:));
end
se_lincom(1,1)=std(beta1_lincom_breps);


betaIFE;
seIFE;
sigma2;





