/*
This program loads in the NLSY data and creates additional variables 
that will be used for sample selection and/or analysis.

User needs to set global "mydatapath" to point to the raw NLSY data.
This program and the analysis are currently set up recursively for 
a directory that has three folders: "./programs", "./mydata", and "./output"
*/


global mydatapath "../mydata"  


*** IMPORT CSV DATASETS AND STORE THEM AS STATA DATASETS
import delimited ${mydatapath}/Time-Inv_Demographics.csv, case(preserve) clear 
save ${mydatapath}/Time-Inv_Demographics.dta, replace

import delimited ${mydatapath}/Time-Inv_Ability.csv, case(preserve) clear
save ${mydatapath}/Time-Inv_Ability.dta, replace

import delimited ${mydatapath}/Time-Variant.csv, case(preserve) clear
save ${mydatapath}/Time-Variant.dta, replace

import delimited ${mydatapath}/CPI_URS_19782017.csv, case(preserve) clear
save ${mydatapath}/CPI_URS_19782017.dta, replace


*** MERGE RAW NLSY DATA TOGETHER
use ${mydatapath}/Time-Inv_Demographics.dta, clear

merge 1:1 caseid_1979 using ${mydatapath}/Time-Inv_Ability.dta, gen(_mergeAbility)

merge 1:m caseid_1979 using ${mydatapath}/Time-Variant.dta, gen(_mergeTimeVar)

sort caseid_1979 year
xtset caseid_1979 year


*** CREATE ADDITIONAL VARIABLES
replace HourlyWage1=. if HourlyWage1==0
*cpi inflation factors
preserve 
use ${mydatapath}/CPI_URS_19782017.dta, clear
gen avg99=avg if year==1999
egen base99=max(avg99)
gen factor99=base99/avg
tempfile adjustment_factors
save "`adjustment_factors'"
restore

merge m:1 year using "`adjustment_factors'", keepusing(factor99) gen(_mergeAdjFactors)
drop if _mergeAdjFactors==2

gen Salary_plusone=Salary+1
gen Salary_cpi=Salary*factor99
gen Salary_plusone_cpi=Salary_plusone*factor99
gen HourlyWage1_cpi=HourlyWage1*factor99

gen log_Salary_cpi=log(Salary_cpi)
gen log_Salary_plusone_cpi=log(Salary_plusone_cpi)
gen log_HourlyWage1_cpi=log(HourlyWage1_cpi)

*quadratic terms, interactions, and birth quarters
gen Education2=Education*Education
gen Age2=Age*Age
gen Age4=Age*Age*Age*Age
gen EducAge=Education*Age

gen BirthQuarter=.
replace BirthQuarter=1 if BirthMonth>=1 & BirthMonth<=3
replace BirthQuarter=2 if BirthMonth>=4 & BirthMonth<=6
replace BirthQuarter=3 if BirthMonth>=7 & BirthMonth<=9
replace BirthQuarter=4 if BirthMonth>=10 & BirthMonth<=12

*identify individuals with negative schooling changes (during time frame of analysis: 1981-2016)
sort caseid_1979 year
by caseid_1979: gen educ_change=Education-Education[_n-1] if _n>1 & !missing(Education) & !missing(Education[_n-1]) & year>=1982

by caseid_1979: egen min_educ_change=min(educ_change)

*turn HourlyWage1 in to dollars instead of cents
replace HourlyWage1=HourlyWage1/100
replace HourlyWage1_cpi=HourlyWage1*factor99
replace log_HourlyWage1_cpi=log(HourlyWage1_cpi)

*create raw proxy-by-year group variables
egen AFQT2xY=group(afqt_2_1981 year)
egen AFQT3xY=group(afqt_3_1981 year)
egen ROTTxY=group(rotter_score_1979 year)
egen ROSENxY=group(rosenberg_esteem_score_1980 year)

*rescale Rotter and Rosenberg variables so that higher scores are "better" (greater internal control or higher self-esteem)
foreach i in 1 2 3 {
	replace rotter_`i'a_1979=1+(2-rotter_`i'a_1979)
}
foreach i in 01 02 04 06 07 {
	replace rosenberg_esteem_0000`i'_1980=1+(4-rosenberg_esteem_0000`i'_1980)
}

replace rotter_score_1979=4+(16-rotter_score_1979)

*create standardized proxy variables
egen afqt2_std=std(afqt_2_1981)
egen afqt3_std=std(afqt_3_1981)
egen rott_std=std(rotter_score_1979)
egen rosen_std=std(rosenberg_esteem_score_1980)
forval i=3/12 {
	egen asvab_`i'_1981_std=std(asvab_`i'_1981)
}
forval i=1/4 {
	egen rotter_`i'a_1979_std=std(rotter_`i'a_1979)
}
foreach i in 01 02 03 04 05 06 07 08 09 10 {
	egen rosenberg_esteem_0000`i'_1980_std=std(rosenberg_esteem_0000`i'_1980)
}

*create demo-by-year variables 
gen hispanic=(Sample_Race==1)
gen married=(Marriage==1)
gen foreign_born=(ForeignBorn==1)
egen FxY = group(Female year)
egen FBxY = group(foreign_born year)
egen BYxY = group(BirthYear year)
egen RxY = group(Sample_Race year)
egen MxY = group(married year)
egen HxY = group(hispanic year)

*identify outlier values
pctile hourlywage_ptiles = HourlyWage1_cpi, nq(100)
xtile hourlywage_percentile = HourlyWage1_cpi, cutpoints(hourlywage_ptiles)

gen notoutlier=(hourlywage_percentile>1 & hourlywage_percentile<100)
replace notoutlier=. if missing(HourlyWage1_cpi)

*identify individuals with abnormally large schooling changes (schooling years change>time years changes)
sort caseid_1979 year
by caseid_1979: gen year_change=year-year[_n-1] if _n>1
gen largechange=(educ_change>year_change & !missing(educ_change) & !missing(year_change))
sort caseid_1979 year
by caseid_1979: egen largechanger=max(largechange)



save ${mydatapath}/NLSYrep_prep.dta, replace

