/*
This program selects the samples for analysis, 
creates a waterfall table showing how the sample 
selection criteria affect the sample size,
creates a figure showing the distribution of 
years in the sample per person, and distribution 
of changes in years of schooling per person and 
per level of schooling.

It also creates several alternative samples.

The so-called "comparison sample" from the paper 
lacks the additional sample restrictions imposed 
by the various alternative samples. It is identified 
by the "insample" variable in this program. The 
alternative samples are identified by the 
"insample_{name}" variables. The "insample_minti15" 
sample is the main sample in the published version 
of the paper. Relative to the "comparison sample" 
it imposes a larger restriction on the minimum 
number of observations per individual. The other 
alternative samples in this program also impose 
the larger restriction.
*/

global mydatapath "../mydata"
global outputpath "../output"


use ${mydatapath}/NLSYrep_prep.dta, clear


*** IDENTIFY SAMPLE
eststo clear

gen insample=0

*no missing variables, year>=1981, age>=16 [BASE SAMPLE]
replace insample=1 if !missing(log_HourlyWage1_cpi) & !missing(Education) & !missing(Age) & !missing(afqt2_std) & !missing(afqt3_std) & !missing(rott_std) & !missing(rosen_std) & !missing(FatherEducation) &!missing(MotherEducation) & !missing(Sibling) & year>=1981 & Age>=16

eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std if insample==1, listwise


*white males
gen insample_altb=insample
replace insample=0 if Female==1 | Sample_Race<3
replace insample_altb=0 if Female==1

eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std if insample==1, listwise


*weeks and hours restrictions from Koop and Tobias (2004)
gen insample_altwh=insample
replace insample=0 if Calendar_Weeks<30 | missing(Calendar_Weeks) | Calendar_Hours<800 | missing(Calendar_Hours)
replace insample_altb=0 if Calendar_Weeks<30 | missing(Calendar_Weeks) | Calendar_Hours<800 | missing(Calendar_Hours)
replace insample_altwh=0 if Calendar_Weeks<40 | missing(Calendar_Weeks) | Calendar_Hours<1200 | missing(Calendar_Hours)

eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std if insample==1, listwise


*no outlier wages
replace insample=0 if notoutlier==0 
replace insample_altb=0 if notoutlier==0 
replace insample_altwh=0 if notoutlier==0

eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std if insample==1, listwise


*no individuals with negative education changes or abnormally large schooling changes
replace insample=0 if min_educ_change<0 | largechanger==1
replace insample_altb=0 if min_educ_change<0 | largechanger==1
replace insample_altwh=0 if min_educ_change<0 | largechanger==1


*minimum T_i
by caseid_1979: egen person_samp_obs=count(insample) if insample==1
replace insample=0 if person_samp_obs==1

gen insample_minti15=insample
replace insample_minti15=0 if person_samp_obs<15

by caseid_1979: egen person_sampaltb_obs=count(insample_altb) if insample_altb==1
replace insample_altb=0 if person_sampaltb_obs<15
by caseid_1979: egen person_sampaltwh_obs=count(insample_altwh) if insample_altwh==1
replace insample_altwh=0 if person_sampaltwh_obs<15



*** CALCULATE COGNITIVE AND NON-COGNITIVE FACTORS FOR FINAL SAMPLE
by caseid_1979: egen firstyear=min(year) if insample ==1
pca asvab_3_1981_std-asvab_12_1981_std if insample==1 & year==firstyear 
predict factor_c, score

pca rotter_1a_1979_std rotter_2a_1979_std rotter_3a_1979_std rotter_4a_1979_std rosenberg_esteem_0000*_std if insample==1 & year==firstyear 
predict factor_n1 factor_n2 factor_n3, score


by caseid_1979: egen firstyear_altb=min(year) if insample_altb ==1
pca asvab_3_1981_std-asvab_12_1981_std if insample_altb==1 & year==firstyear_altb 
predict factor_c_altb, score

pca rotter_1a_1979_std rotter_2a_1979_std rotter_3a_1979_std rotter_4a_1979_std rosenberg_esteem_0000*_std if insample_altb==1 & year==firstyear_altb
predict factor_n1_altb factor_n2_altb factor_n3_altb, score


by caseid_1979: egen firstyear_altwh=min(year) if insample_altwh ==1
pca asvab_3_1981_std-asvab_12_1981_std if insample_altwh==1 & year==firstyear_altwh
predict factor_c_altwh, score

pca rotter_1a_1979_std rotter_2a_1979_std rotter_3a_1979_std rotter_4a_1979_std rosenberg_esteem_0000*_std if insample_altwh==1 & year==firstyear_altwh
predict factor_n1_altwh factor_n2_altwh factor_n3_altwh, score


by caseid_1979: egen firstyear_minti15=min(year) if insample_minti15 ==1
pca asvab_3_1981_std-asvab_12_1981_std if insample_minti15==1 & year==firstyear_minti15
predict factor_c_minti15, score

pca rotter_1a_1979_std rotter_2a_1979_std rotter_3a_1979_std rotter_4a_1979_std rosenberg_esteem_0000*_std if insample_minti15==1 & year==firstyear_minti15
predict factor_n1_minti15 factor_n2_minti15 factor_n3_minti15, score


eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std factor_c factor_n1 factor_n2 factor_n3 if insample==1, listwise

eststo: estpost summarize HourlyWage1_cpi log_HourlyWage1_cpi Education Age MotherEducation FatherEducation Sibling  afqt2_std asvab_3_1981_std-asvab_12_1981_std rott_std rosen_std factor_c_minti15 factor_n1_minti15 factor_n2_minti15 factor_n3_minti15 if insample_minti15==1, listwise



*** WATERFALL TABLE
label var HourlyWage1_cpi "Hourly wage inflation-adjusted"
label var log_HourlyWage1_cpi "Log hourly wage inflation-adjusted"
label var Education "Years of school"
label var Age "Age"
label var MotherEducation "Mother's years of school"
label var FatherEducation "Father's years of school"
label var Sibling "Number of siblings"
label var afqt2_std "AFQT standardized"
label var asvab_3_1981_std "ASVAB standardized - science"
label var asvab_4_1981_std "ASVAB standardized - arithmetic reasoning"
label var asvab_5_1981_std "ASVAB standardized - word knowledge"
label var asvab_6_1981_std "ASVAB standardized - paragraph comprehension"
label var asvab_7_1981_std "ASVAB standardized - numeric operations"
label var asvab_8_1981_std "ASVAB standardized - coding speed"
label var asvab_9_1981_std "ASVAB standardized - auto and shop information"
label var asvab_10_1981_std "ASVAB standardized - mathmematical knowledge"
label var asvab_11_1981_std "ASVAB standardized - mechanical comprehension"
label var asvab_12_1981_std "ASVAB standardized - electronic information"
label var rott_std "Rotter locus of control standardized"
label var rosen_std "Rosenberg self-esteem standardized"
label var factor_c "Cognitive factor"
label var factor_n1 "Noncognitive factor 1"
label var factor_n2 "Noncognitive factor 2"
label var factor_n3 "Noncognitive factor 3"
label var factor_c_minti15 "Cognitive factor - minTi=15"
label var factor_n1_minti15 "Noncognitive factor 1 - minTi=15"
label var factor_n2_minti15 "Noncognitive factor 2 - minTi=15"
label var factor_n3_minti15 "Noncognitive factor 3 - minTi=15"
esttab using ${outputpath}/waterfall.csv, cells("mean(fmt(3)) sd(fmt(3))") mtitles("No missing variables and age 16+" "White males" "Weeks and hours worked restrictions" "No outlier wages" "No negative or abnormal schooling changes" "min(Ti)=15") nodepvar label replace



*** CHECK DISTRIBUTION OF YEARS IN THE SAMPLE PER PERSON
preserve
keep if insample ==1
bys caseid_1979: gen NumberYears = _N
collapse NumberYears, by (caseid_1979)
sum NumberYears
tabulate NumberYears
hist NumberYears, percent xtitle("Number of Years")
*hist NumberYears, frequency
tempfile yearsinsamp
save `yearsinsamp'
graph export ${outputpath}/distribution_yearsinsample.png, replace
restore



*** CHECK NUMBER OF CHANGES PER PERSON/YEAR OF SCHOOLING
*per person
preserve
putexcel set ${outputpath}/distribution_changesperson.xlsx, replace
gen educ_change_event=(educ_change==1 | educ_change==2)
keep if insample_minti15 ==1
collapse (sum) educ_change_event, by(caseid_1979)
sum educ_change_event
tabulate educ_change_event, matcell(freq) matrow(names)
putexcel A1=("Number of Changes") B1=("Number of Individuals") C1=("Percent") 
putexcel A2=matrix(names) B2=matrix(freq) C2=matrix(freq/r(N)) 
restore
***per year of school
putexcel set ${outputpath}/distribution_changeslevel.xlsx, replace
tab Education if insample_minti15==1 & (educ_change==1 | educ_change==2), matcell(freq) matrow(names)
putexcel A1=("Education Level After Change") B1=("Number of Changes") C1=("Percent") 
putexcel A2=matrix(names) B2=matrix(freq) C2=matrix(freq/r(N)) 




save ${mydatapath}/NLSYrep_doa.dta, replace

